import React, { createContext, useState, useRef, useEffect, useCallback } from 'react';
import { getTodo, addTodo, delTodo, uptTodo } from '../../API/Api';
import { Header } from '../../Layout/card/Index';
import styles from './styles.module.css';
import List from '../../todo/List/Index';

export const MyTask = createContext();

export const Input = () => {
  const [task, setTask] = useState("");
  const [store, setStore] = useState([]);
  const [editIndex, setEditIndex] = useState(null);
  const [loading, setLoading] = useState(false);
  const [isNewTodoAdded, setIsNewTodoAdded] = useState(false);
  const editableRef = useRef(null);
  const debounceTimeout = useRef(null);
  const lastClickEvent = useRef(null);

  const fetchTodos = useCallback(async () => {
    try {
      setLoading(true);
      const res = await getTodo();
      setStore(res.data || []);
    } catch (error) {
      console.error("Failed to fetch todos:", error);
    } finally {
      setLoading(false);
    }
  }, []);

  useEffect(() => {
    fetchTodos();
  }, [fetchTodos]);

  const placeCaretAtClick = () => {
    const el = editableRef.current;
    const event = lastClickEvent.current;

    if (el && event) {
      let range, sel;
      if (document.caretRangeFromPoint) {
        range = document.caretRangeFromPoint(event.clientX, event.clientY);
      } else if (document.caretPositionFromPoint) {
        const pos = document.caretPositionFromPoint(event.clientX, event.clientY);
        range = document.createRange();
        range.setStart(pos.offsetNode, pos.offset);
      }

      if (range) {
        sel = window.getSelection();
        sel.removeAllRanges();
        sel.addRange(range);
      }
    }
  };

  const handleEdit = (e) => {
    const newValue = e.currentTarget.textContent;

    setStore(prevStore => {
      const updatedStore = [...prevStore];
      if (updatedStore[editIndex]) {
        updatedStore[editIndex] = {
          ...updatedStore[editIndex],
          task: newValue
        };
      }
      return updatedStore;
    });

    if (debounceTimeout.current) {
      clearTimeout(debounceTimeout.current);
    }

    debounceTimeout.current = setTimeout(() => {
      const currentItem = store[editIndex];
      if (currentItem) {
        uptTodo(currentItem.id, {
          id: currentItem.id,
          task: newValue,
          completed: currentItem.completed,
        }).catch(console.error);
      }
    }, 1000);
  };

  const handleChange = (e) => setTask(e.target.value);

  const startEdit = (index, clickEvent = null) => {
    setEditIndex(index);
    lastClickEvent.current = clickEvent;

    setTimeout(() => {
      if (editableRef.current) {
        const actualText = store[index]?.task || "";
        editableRef.current.innerText = actualText;
        editableRef.current.focus();
        placeCaretAtClick();
      }
    }, 0);
  };

  const endEdit = () => {
    setEditIndex(null);
  };

  const Add = async () => {
    if (task.trim() === '') return;
    try {
      setIsNewTodoAdded(true);
      const res = await addTodo(task);
      setStore(prev => [...prev, res.data]);
      setTask("");
    } catch (error) {
      console.error("Error adding todo:", error);
    } finally {
      setTimeout(() => setIsNewTodoAdded(false), 500);
    }
  };

  const Delete = async (id) => {
    const confirmDelete = window.confirm("Are you sure you want to delete?");
    if (confirmDelete) {
      try {
        await delTodo(id);
        setStore(prev => prev.filter(item => item.id !== id));
      } catch (error) {
        console.error("Error deleting todo:", error);
      }
    }
  };

  const toggleComplete = async (index) => {
    try {
      const updated = [...store];
      updated[index].completed = !updated[index].completed;
      setStore(updated);

      await uptTodo(updated[index].id, updated[index]);
    } catch (error) {
      console.error("Error toggling complete:", error);
      fetchTodos();
    }
  };

  const renderEditableContent = () => (
    <div
      ref={editableRef}
      contentEditable
      suppressContentEditableWarning
      onBlur={endEdit}
      onInput={handleEdit}
      className={`${styles.task} ${store[editIndex]?.completed ? 'completed' : ''}`}
      style={{
        display: 'inline-block',
        width: '100%',
        boxSizing: 'border-box',
        outline: 'none',
        whiteSpace: 'pre-wrap',
        // margin: 0,
        marginscreenLeft:'0.2rem',
        padding: '4px 0',
        userSelect: 'text'
      }}
    />
  );

  return (
    <MyTask.Provider value={{ store, setStore }}>
      <div className="todo-container">
        <Header />
        <List
          store={store}
          editIndex={editIndex}
          renderEditableContent={renderEditableContent}
          startEdit={(index, e) => startEdit(index, e)}
          toggleComplete={toggleComplete}
          Delete={Delete}
          isNewTodoAdded={isNewTodoAdded}
        />
        <form onSubmit={(e) => e.preventDefault()}>
          <div className={styles.merge}>
            <input
              type="text"
              className={styles.input}
              placeholder="Enter Items"
              value={task}
              onChange={handleChange}
              disabled={loading}
            />
            <button
              onClick={Add}
              className={styles.button}
              disabled={loading || !task.trim()}
            >
              Submit
            </button>
          </div>
        </form>
      </div>
    </MyTask.Provider>
  );
};
